import { useEffect, useRef, useState } from 'react';
import GridLayout from 'react-grid-layout';
import Panel from '../components/panels/panel';
import { RawJSONVis } from '@graphpolaris/shared/lib/vis/rawjsonvis/rawjsonvis';
import SemanticSubstrates from '@graphpolaris/shared/lib/vis/semanticsubstrates/semanticsubstrates';
import { Schema } from '@graphpolaris/shared/lib/schema/panel';
import {
  Query2BackendQuery,
  QueryBuilder,
} from '@graphpolaris/shared/lib/querybuilder';
import {
  assignNewGraphQueryResult,
  useAppDispatch,
} from '@graphpolaris/shared/lib/data-access/store';
import { Navbar } from '../components/navbar/navbar';
import { VisualizationPanel } from './panels/Visualization';
import {
  readInSchemaFromBackend,
  useAuthorization,
  useAuthorizationCache,
  useDatabaseAPI,
  useQueryAPI,
  useQuerybuilderGraph,
  useQuerybuilderGraphology,
  useSchemaAPI,
  useSessionCache,
} from '@graphpolaris/shared/lib/data-access';
import LoginScreen from '../components/login/loginScreen';
import { WebSocketHandler } from '@graphpolaris/shared/lib/data-access/socket';
import Broker from '@graphpolaris/shared/lib/data-access/socket/broker';
import { SchemaFromBackend } from '@graphpolaris/shared/lib/model/backend';
import { domain } from '../environments/variables';
import { QueryMultiGraphExport } from '@graphpolaris/shared/lib/querybuilder/graph/graphology/utils';
import { GraphQueryResultFromBackend } from '@graphpolaris/shared/lib/data-access/store/graphQueryResultSlice';

export function App() {
  const { AuthorizeFromCache, auth } = useAuthorization(domain);
  const api = useDatabaseAPI(domain);
  const api_schema = useSchemaAPI(domain);
  const api_query = useQueryAPI(domain);
  const dispatch = useAppDispatch();
  const session = useSessionCache();
  const query = useQuerybuilderGraph();
  const ws = useRef(new WebSocketHandler(domain));

  useEffect(() => {
    // Default
    AuthorizeFromCache();
    Broker.instance().subscribe(
      (data: SchemaFromBackend) => dispatch(readInSchemaFromBackend(data)),
      'schema_result'
    );

    Broker.instance().subscribe(
      (data: GraphQueryResultFromBackend) =>
        dispatch(assignNewGraphQueryResult(data)),
      'query_result'
    );

    return () => {
      Broker.instance().unSubscribeAll('schema_result');
      Broker.instance().unSubscribeAll('query_result');
    };
  }, []);

  useEffect(() => {
    // New active database
    if (session.currentDatabase) {
      ws.current.useToken(auth.accessToken).connect(() => {
        api_schema.RequestSchema(session.currentDatabase);
      });
    }
  }, [session.currentDatabase]);

  useEffect(() => {
    // Newly (un)authorized
    console.log(auth.authorized);
    if (auth.authorized) {
      api.GetAllDatabases({ updateSessionCache: true });
    }
  }, [auth.authorized]);

  useEffect(() => {
    // New query
    if (session?.currentDatabase && query) {
      api_query.execute(Query2BackendQuery(session.currentDatabase, query));
    }
  }, [query]);

  return (
    <div className="h-screen w-screen">
      {!auth.authorized && <LoginScreen />}
      <div
        className={
          'flex h-screen w-screen overflow-hidden ' +
          (!auth.authorized ? 'blur-sm pointer-events-none ' : '')
        }
      >
        <div className="h-full relative flex flex-col flex-1 overflow-y-auto overflow-x-hidden">
          <div className="h-fit flex-grow-0">
            <Navbar />
          </div>
          <main className="h-full flex gap-4 p-10">
            <div className="w-full basis-[33%]">
              <Panel content="Schema">
                <Schema />
              </Panel>
            </div>
            <div className="w-full basis-[67%] flex flex-col gap-4">
              <div className="h-full basis-[67%] overflow-y-clip">
                <VisualizationPanel />
              </div>
              <div className="h-full basis-[33%] flex-grow-0">
                <Panel content="Query Panel">
                  <QueryBuilder />
                </Panel>
              </div>
            </div>
          </main>
        </div>
      </div>
      {/*<GridLayout*/}
      {/*  className="layout"*/}
      {/*  cols={10}*/}
      {/*  rowHeight={30}*/}
      {/*  width={window.innerWidth}*/}
      {/*>*/}
      {/*  <div*/}
      {/*    key="schema-panel"*/}
      {/*    data-grid={{ x: 0, y: 0, w: 3, h: 30, maxW: 5, isDraggable: false }}*/}
      {/*  >*/}
      {/*    <Panel content="Schema Panel" color="red">*/}
      {/*      <Schema />*/}
      {/*    </Panel>*/}
      {/*  </div>*/}
      {/*  <div*/}
      {/*    key="query-panel"*/}
      {/*    data-grid={{ x: 3, y: 20, w: 5, h: 10, maxH: 20, isDraggable: false }}*/}
      {/*  >*/}
      {/*    <Panel content="Query Panel" color="blue">*/}
      {/*      <QueryBuilder />*/}
      {/*    </Panel>*/}
      {/*  </div>*/}
      {/*  <div*/}
      {/*    key="visualisation-panel"*/}
      {/*    data-grid={{ x: 3, y: 0, w: 7, h: 20, isDraggable: false }}*/}
      {/*  >*/}
      {/*    <VisualizationPanel />*/}
      {/*  </div>*/}
      {/*  <div*/}
      {/*    key="history-panel"*/}
      {/*    data-grid={{ x: 8, y: 20, w: 2, h: 10, isDraggable: false }}*/}
      {/*  >*/}
      {/*    <Panel content="History Channel" color="purple"></Panel>*/}
      {/*  </div>*/}
      {/*</GridLayout>*/}
    </div>
  );
}

export default App;
